﻿using Moq;
using NUnit.Framework;
using Playnite;
using Playnite.Common;
using Playnite.Database;
using Playnite.SDK.Models;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Playnite.Tests.Database
{
    [TestFixture]
    public class GameDatabasePlatformsTests
    {
        [Test]
        public void PlatformGeneratorTest()
        {
            using (var temp = TempDirectory.Create())
            {
                var db = new GameDatabase(temp.TempPath);
                db.OpenDatabase();
                var platforms = db.Platforms;
                CollectionAssert.IsNotEmpty(db.Platforms);
                db.Platforms.Remove(platforms);
                CollectionAssert.IsEmpty(db.Platforms);
                db.Platforms.Add(new Platform("Test"));
                db.Platforms.Add(new Platform("Test2"));
                Assert.AreEqual(2, db.Platforms.Count);
            }
        }

        [Test]
        public void PlatformRemovalTest()
        {
            using (var temp = TempDirectory.Create())
            {
                var db = new GameDatabase(temp.TempPath);
                db.OpenDatabase();              
                var platform = new Platform("Test");
                db.Platforms.Add(platform);
                var game = new Game("Test")
                {
                    PlatformId = platform.Id
                };

                db.Games.Add(game);
                db.Platforms.Remove(platform);
                var dbGame = db.Games[game.Id];
                Assert.AreEqual(Guid.Empty, dbGame.PlatformId);
                Assert.IsNull(db.Platforms.FirstOrDefault(a => a.Name == "Test"));            
            }
        }

        [Test]
        public void PcPlatformAutoAssignTest()
        {
            using (var temp = TempDirectory.Create())
            {
                var db = new GameDatabase(temp.TempPath);
                db.OpenDatabase();

                // Remove PC platform first, might get autogenerated with clean DB
                var pl = db.Platforms.FirstOrDefault(a => a.Name == "PC");
                if (pl != null)
                {
                    db.Platforms.Remove(pl);
                    Assert.IsNull(db.Platforms.FirstOrDefault(a => a.Name == "PC"));
                }

                var game = new Game()
                {
                    GameId = "testid",
                    Name = "Test Game"
                };

                db.Games.Add(game);
                db.AssignPcPlatform(game);
                var pcPlatform = db.Platforms.FirstOrDefault(a => a.Name == "PC");
                Assert.IsNotNull(pcPlatform);
                Assert.AreEqual(game.PlatformId, pcPlatform.Id);
            }
        }
    }
}
